import React from "react";
import { useRef, useEffect, useState } from "react";
import { useTheme } from "../hooks/useTheme";
import { tokenize } from "../utils/tokenizer";
import { darkTheme, lightTheme } from "../themes";
import { ChevronDown, ChevronUp, Copy, Check } from "lucide-react";
import type { HighlighterProps, Token } from "../types";

export const SyntaxHighlighter: React.FC<HighlighterProps> = ({
  code,
  language,
  showLineNumbers = true,
  wrapLines = false,
  maxHeight,
  expandable = false,
  className = "",
  style = {},
  customBackgroundColor,
  showCopyButton = true,
}) => {
  const { theme } = useTheme();
  const codeRef = useRef<HTMLPreElement>(null);
  const [isExpanded, setIsExpanded] = useState(!expandable);
  const [isCopied, setIsCopied] = useState(false);

  // Get dark mode state directly from DOM
  const isDarkMode = document.documentElement.classList.contains("dark");
  const currentTheme = isDarkMode ? darkTheme : lightTheme;

  useEffect(() => {
    if (codeRef.current) {
      codeRef.current.innerHTML = "";
      const tokens = tokenize(code, language);
      if (tokens.length === 0) return;

      let lineNumber = 1;
      let lineContent = "";

      tokens.forEach((token: Token) => {
        if (showLineNumbers && (lineNumber === 1 || lineContent === "")) {
          const lineNumberSpan = document.createElement("span");
          lineNumberSpan.textContent = `${lineNumber} `;
          lineNumberSpan.className = "line-number select-none";
          lineNumberSpan.style.color = currentTheme.lineNumber;
          lineNumberSpan.style.marginRight = "1em";
          lineNumberSpan.style.opacity = "0.5";
          codeRef.current?.appendChild(lineNumberSpan);
        }

        const span = document.createElement("span");
        span.textContent = token.content;
        span.className = `token ${token.type}`;
        span.style.color = currentTheme[token.type as keyof typeof currentTheme] || currentTheme.text;

        if (token.content.includes("\n")) {
          const lines = token.content.split("\n");
          lines.forEach((line, index) => {
            if (index > 0) {
              codeRef.current?.appendChild(document.createElement("br"));
              if (showLineNumbers) {
                lineNumber++;
                const lineNumberSpan = document.createElement("span");
                lineNumberSpan.textContent = `${lineNumber} `;
                lineNumberSpan.className = "line-number select-none";
                lineNumberSpan.style.color = currentTheme.lineNumber;
                lineNumberSpan.style.marginRight = "1em";
                lineNumberSpan.style.opacity = "0.5";
                codeRef.current?.appendChild(lineNumberSpan);
              }
            }
            if (line) {
              const lineSpan = document.createElement("span");
              lineSpan.textContent = line;
              lineSpan.className = `token ${token.type}`;
              lineSpan.style.color = currentTheme[token.type as keyof typeof currentTheme] || currentTheme.text;
              codeRef.current?.appendChild(lineSpan);
            }
          });
          lineContent = "";
        } else {
          codeRef.current?.appendChild(span);
          lineContent += token.content;
        }
      });
    }
  }, [code, language, isDarkMode, showLineNumbers]);

  const toggleExpand = () => {
    setIsExpanded(!isExpanded);
  };

  const copyToClipboard = async () => {
    if (codeRef.current) {
      try {
        const text = code; // Use original code instead of innerText to preserve formatting
        await navigator.clipboard.writeText(text);
        setIsCopied(true);
        setTimeout(() => setIsCopied(false), 2000);
      } catch (err) {
        console.error("Failed to copy code:", err);
      }
    }
  };

  const defaultClassName = "syntax-highlighter relative font-mono text-sm";
  const combinedClassName = className
    ? `${defaultClassName} ${className}`
    : defaultClassName;

  return (
    <div className={combinedClassName} style={style}>
      <div
        className="syntax-highlighter-content relative"
        style={{
          maxHeight: isExpanded ? "none" : maxHeight,
          transition: "max-height 0.2s ease-in-out",
        }}
      >
        <pre
          ref={codeRef}
          className={`language-${language} p-4 m-0 overflow-x-auto`}
          style={{
            whiteSpace: wrapLines ? "pre-wrap" : "pre",
          }}
        />

        {showCopyButton && (
          <button
            onClick={copyToClipboard}
            className="absolute top-2 right-2 p-2 rounded-md 
                     bg-white/10 hover:bg-white/20 dark:bg-black/10 dark:hover:bg-black/20 
                     transition-colors duration-200"
            aria-label="Copy code"
          >
            {isCopied ? (
              <Check className="w-4 h-4 text-green-500" />
            ) : (
              <Copy className="w-4 h-4 text-gray-500 dark:text-gray-400" />
            )}
          </button>
        )}
      </div>

      {expandable && (
        <button
          onClick={toggleExpand}
          className="w-full flex items-center justify-center p-2 
                   hover:bg-gray-100 dark:hover:bg-gray-800 
                   transition-colors duration-200"
        >
          {isExpanded ? (
            <>
              <ChevronUp className="w-4 h-4 mr-2" />
              <span>Collapse</span>
            </>
          ) : (
            <>
              <ChevronDown className="w-4 h-4 mr-2" />
              <span>Expand</span>
            </>
          )}
        </button>
      )}

      <style >{`
        .syntax-highlighter {
          --scrollbar-width: 8px;
          --scrollbar-height: 8px;
          --scrollbar-track-color: transparent;
          --scrollbar-thumb-color: ${isDarkMode ? 'rgba(75, 85, 99, 0.5)' : 'rgba(156, 163, 175, 0.5)'};
          --scrollbar-thumb-hover-color: ${isDarkMode ? 'rgba(75, 85, 99, 0.8)' : 'rgba(156, 163, 175, 0.8)'};
        }

        .syntax-highlighter ::-webkit-scrollbar {
          width: var(--scrollbar-width);
          height: var(--scrollbar-height);
        }

        .syntax-highlighter ::-webkit-scrollbar-track {
          background: var(--scrollbar-track-color);
        }

        .syntax-highlighter ::-webkit-scrollbar-thumb {
          background-color: var(--scrollbar-thumb-color);
          border-radius: calc(var(--scrollbar-width) / 2);
        }

        .syntax-highlighter ::-webkit-scrollbar-thumb:hover {
          background-color: var(--scrollbar-thumb-hover-color);
        }

        .token {
          transition: color 0.2s ease;
        }
      `}</style>
    </div>
  );
};
