// ThemeContext.tsx
import React, { createContext, useState, useEffect } from "react";
import type { Theme, ThemeContextType } from "../types";
import { lightTheme, darkTheme } from "../themes";

export const ThemeContext = createContext<ThemeContextType>({
  theme: lightTheme,
  setTheme: () => {},
  isDarkMode: false,
  setIsDarkMode: () => {},
});

export const ThemeProvider: React.FC<{ 
  children: React.ReactNode; 
  initialIsDarkMode?: boolean 
}> = ({
  children,
  initialIsDarkMode = false,
}) => {
  const [theme, setTheme] = useState<Theme>(initialIsDarkMode ? darkTheme : lightTheme);
  const [isDarkMode, setIsDarkMode] = useState<boolean>(initialIsDarkMode);

  useEffect(() => {
    // Watch for .dark class on html element instead of media query
    const observer = new MutationObserver((mutations) => {
      mutations.forEach((mutation) => {
        if (mutation.attributeName === 'class') {
          const isDark = document.documentElement.classList.contains('dark');
          setIsDarkMode(isDark);
          setTheme(isDark ? darkTheme : lightTheme);
        }
      });
    });

    observer.observe(document.documentElement, {
      attributes: true,
      attributeFilter: ['class'],
    });

    // Initial check
    const isDark = document.documentElement.classList.contains('dark');
    setIsDarkMode(isDark);
    setTheme(isDark ? darkTheme : lightTheme);

    return () => observer.disconnect();
  }, []);

  return (
    <ThemeContext.Provider value={{ theme, setTheme, isDarkMode, setIsDarkMode }}>
      {children}
    </ThemeContext.Provider>
  );
};
